; ***** BEGIN LICENSE BLOCK *****
; * Copyright (C) 2007  Alexey Gladkov <legion@altlinux.org>
; *
; * This program is free software; you can redistribute it and/or modify
; * it under the terms of the GNU General Public License as published by
; * the Free Software Foundation; either version 2 of the License, or
; * (at your option) any later version.
; *
; * This program is distributed in the hope that it will be useful,
; * but WITHOUT ANY WARRANTY; without even the implied warranty of
; * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; * GNU General Public License for more details.
; *
; * You should have received a copy of the GNU General Public License
; * along with this program; if not, write to the Free Software
; * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
; ***** END LICENSE BLOCK *****

(document:surround "/std/frame")
(document:surround "/std/functions")

;(document:envelop with-attributes (password1 password2))
;(document:envelop with-container-presentations ((password-box '/root/passwd)))

(define sections (make-cell '()))


(define (get-current-type)
  (let ((curr (sects current)))
    (if (or (< curr 0 ) (>= curr (sects count))) "image"
      (woo-get-option (list-ref (cell-ref sections) curr) 'type))))

(define (make-view cmd)
  (cons (woo-get-option cmd 'name)
        (if (woo-get-option cmd 'default #f) "on" "off")))

(define (get-section-params)
	(button-apply activity #f)
	(let ((info (woo-read-first "/lilo/sections" 'section (sects text))))
          (section-label   text  (woo-get-option info 'label ""))
	  (if (string= (get-current-type) "other")
	    (begin
	      (section-image   visibility #f)
	      (section-initrd  visibility #f)
	      (section-root    visibility #f)
	      (section-append  visibility #f)
              (section-other   visibility #t text (woo-get-option info 'other ""))
              (section-alias   visibility #t text (woo-get-option info 'alias ""))
              (section-boot-as visibility #t text (woo-get-option info 'boot_as ""))
              (section-table   visibility #t text (woo-get-option info 'table))
	      (section-type-label text (_"Other loader"))
	      (section-image-label   visibility #f)
	      (section-initrd-label  visibility #f)
	      (section-root-label    visibility #f)
	      (section-append-label  visibility #f)
              (section-other-label   visibility #t)
              (section-alias-label   visibility #t)
              (section-boot-as-label visibility #t)
              (section-table-label   visibility #t)
	    )
	    (begin
              (section-other   visibility #f)
              (section-alias   visibility #f)
              (section-boot-as visibility #f)
              (section-table   visibility #f)
              (section-other-label   visibility #f)
              (section-alias-label   visibility #f)
              (section-boot-as-label visibility #f)
              (section-table-label   visibility #f)
	      (section-image   visibility #t text (woo-get-option info 'image ""))
	      (section-initrd  visibility #t text (woo-get-option info 'initrd ""))
	      (section-root    visibility #t text (woo-get-option info 'root ""))
	      (section-append  visibility #t text (woo-get-option info 'addappend ""))
	      (section-type-label text (_"Linux kernel"))
	      (section-image-label   visibility #t)
	      (section-initrd-label  visibility #t)
	      (section-root-label    visibility #t)
	      (section-append-label  visibility #t)
            ))))

(define (update-sections-list)
	(sects rows (map make-view (cell-ref sections))))

(define (init-sections)
	(cell-set! sections (woo-catch/message (thunk (woo-list "/lilo/sections"))))
	(update-sections-list))

(define (reload-sections)
    (let ((curr (sects current)))
	(init-sections)
	(sects current
	 (if (or (< curr 0) (>= curr (sects count))) 0 curr))
	(get-section-params)
	))

(define (add-section type)
	(cell-set! sections (append (cell-ref sections)
	  `(("" name ,(_ "<NEW>") type ,type default #f))))
	(update-sections-list)
	(sects current (- (sects count) 1))
	(get-section-params))

(define (remove-section)
  (if (and
    (>= (sects current) 0)
    (< (sects current) (sects count)))
    (begin
      (if (not (string= (sects text) (_ "<NEW>")))
        (woo-catch/message (thunk (woo-write "/lilo/section-remove" 'label (sects text)))))

      (cell-set! sections (list-remove (cell-ref sections) (sects current)))
      (init-sections)
      (sects current 0)
      (get-section-params))))

(define (real-write-section)
  (woo-write "/lilo/section"
    'type      (get-current-type)
    'oldlabel  (if (string= (sects text) (_ "<NEW>")) "" (sects text))
    'label     (section-label text)
    'image     (section-image text)
    'initrd    (section-initrd text)
    'root      (section-root text)
    'addappend (section-append text)
    'other     (section-other text)
    'alias     (section-alias text)
    'boot_as   (section-boot-as text)
    'table     (section-table text)))

;;FIXME: fix catch usage (all calls inside woo-catch/message)
(define (write-section)
  (and (woo-catch/message (thunk (real-write-section)))
       (button-apply activity #f)
       (reload-sections)))


(define (get-global-params)
  (let ((info (woo-read-first '/lilo/globals)))
	(global-boot text (woo-get-option info 'boot ""))
	(global-timeout text (woo-get-option info 'timeout ""))
	(global-append text (woo-get-option info 'append ""))
	(config-only state (woo-get-option info 'configonly #f))
	(global-install value (woo-get-option info 'install "text"))
	(global-raid-extra-boot value (woo-get-option info 'raid_extra_boot ""))))

(define (real-write-globals)
    (woo-write "/lilo/globals"
       'target   "globals"
       'boot     (global-boot text)
       'append   (global-append text)
       'timeout  (global-timeout text)
       'install  (global-install value)
       'extraboot (global-raid-extra-boot value)
       'method "insecure"
       'confonly (config-only state)))

(define (write-globals)
  (woo-catch/message (thunk
    (real-write-globals)
)))

(define (write-n-commit)
  (woo-catch/message (thunk
    (real-write-globals)
    (woo-write "/lilo/commit" 'confonly (config-only state))
)))

(define (write-reset)
	(woo-catch/message (thunk
	  (woo-try "reset" "/lilo" 'part "globals")
	  (woo-try "reset" "/lilo" 'part "sections")))
	(get-global-params)
	(reload-sections))

(define (change url)
	(woo-catch/message (thunk (write-globals)))
	(frame:replace url))

(define (info-changed)
	(button-apply activity #t))

;;;

(gridbox columns "0;100;0;0;0" align "top"
  (document:id config-only
    (checkbox (_ "Don't install bootloader") state #f))
  (spacer)
  (button (_ "Simple mode")
    (when clicked (write-globals) (change "/lilo")))
  (button (_ "Expert mode") activity #f)
  (button (_ "Edit lilo.conf")
    (when clicked (write-globals) (change "/lilo/edit-source")))
)

(separator)
(label (string-append "<b>" (_ "Common settings") "</b>"))

(gridbox columns "0;50;0;50"

  (label (_ "Install boot loader on:") align 'right)
  (document:id global-boot (edit ""))
  (label (_ "Raid-extra-boot:")  align 'right)
  (document:id global-raid-extra-boot (combobox enumref "lilo/raid_extra_boot_list"))

  (label (_ "Interface type:")  align 'right)
  (document:id global-install (combobox enumref "lilo/install_list"))
  (label (_ "Interface timeout:")    align 'right)
  (document:id global-timeout (edit ""))

  (label (_ "Common kernel parameters:") align 'right)
  (document:id global-append  (edit colspan 3 ""))
  )

(separator)
(label (string-append "<b>" (_ "Boot variants") "</b>"))

(gridbox columns "0;100"
  (document:id sects (listbox rowspan 2 (when selected (get-section-params))))

  (gridbox rowspan 2 columns "1;99" align "top"
    (spacer) (document:id section-type-label   (label))
    (label (_ "Label:") align 'right) (document:id section-label   (edit "" (when changed (info-changed))))

    (document:id section-image-label   (label (_ "Kernel image:")   align 'right)) (document:id section-image   (edit "" (when changed (info-changed))))
    (document:id section-initrd-label  (label (_ "Initrd:")  align 'right)) (document:id section-initrd  (edit "" (when changed (info-changed))))
    (document:id section-root-label    (label (_ "Root device:")    align 'right)) (document:id section-root    (edit "" (when changed (info-changed))))
    (document:id section-append-label  (label (_ "Additional parameters:")  align 'right)) (document:id section-append  (edit "" (when changed (info-changed))))

    (document:id section-other-label   (label (_ "Device:")  align 'right)) (document:id section-other   (edit "" (when changed (info-changed))))
    (document:id section-alias-label   (label (_ "Alias:")   align 'right)) (document:id section-alias   (edit "" (when changed (info-changed))))
    (document:id section-boot-as-label (label (_ "Boot-as:") align 'right)) (document:id section-boot-as (edit "" (when changed (info-changed))))
    (document:id section-table-label   (label (_ "Table:")   align 'right)) (document:id section-table   (edit "" (when changed (info-changed))))
    (spacer)
    (gridbox columns "0;0;50;0;0"
      (document:id button-apply (button (_ "Save") activity #f (when clicked (write-section))))
      (button (_ "Reset") (when clicked (get-section-params) ))
      (spacer)
      (button (_ "Set default")
        (when clicked
          (woo-catch/message
            (thunk (woo-write "/lilo/globals" 'target "default" 'default (sects text))))
          (reload-sections)))
      (button (_ "Remove") (when clicked (remove-section)))
    )
  )
)

(gridbox columns "0;0;100" align "top"
  (button (_ "Add Linux kernel") (when clicked (add-section "image")))
  (button (_ "Add other loader") (when clicked (add-section "other")))
)

(separator)


;;;;


(or (global 'frame:next)
    (gridbox columns "50;0;0;50" align "top"
        (spacer)
        (button (_ "Apply") (when clicked (write-n-commit)))
        (button (_ "Reset") (when clicked (write-reset)))
        (spacer)))

(and (global 'frame:next)
	(frame:on-next (thunk (or (write-n-commit) 'cancel))))

(document:root (when loaded
			(and (global 'frame:next) (frame:back-activity #f))
			(reload-sections)
			(get-global-params) ))
